% opt should contain the diags of the kernel to speed up the learning
function [vModelResults,vClassifyResults] = crossvalidation_pvm(fKernelPtr, L, v, varargin)
%CROSSVAL K-fold cross validation.

error(nargchk(3, 5, nargin))
[N] = numel(L);
if nargin < 2
  v = [];
end

if nargin < 4
  opt = [];
end

if(~isempty(varargin))
    opt = varargin{1};
    C   = 10;
    if(numel(varargin)>1)
        C = varargin{2};
    end
end
if isempty(v)
  V = 1:N;
  v = N;
elseif length(v) == 1
  if ~isa(v, 'double') | ~isreal(v) | v <= 0 | round(v) ~= v
    error('V must be a positive, non-zero integer.')
  elseif v > N
    error('V must be less than the number of observations.')
  elseif v == N
    V = 1:N;
  else
    w = v*fix(N/v);
    r = randperm(v);
    V(randperm(N)) = [reshape(repmat(1:v, w/v, 1), w, 1); ...
		      r(1:N-w)'];
  end
else
  if ~isa(v, 'double') | ~isreal(v) | prod(size(v)) ~= length(v) | ...
	any(round(v) ~= v | v <= 0 | isinf(v))
    error(['Cross-validation set indeces V must be a vector of' ...
	   ' positive, finite, non-zero integers.'])
  elseif length(v) ~= N
    error(['Length of set indeces V must be same as number of' ...
	   ' observations in X and K.'])
  elseif ~all(sum(sparse(1:N, v, 1)))
    error('Set indeces V may not have empty groups.')
  end
  
  V = v;
  v = max(V);
end
  
c = zeros(N, 1);
vModelResults    = [];
vClassifyResults = [];

for i = 1:v
  fprintf('*')
  s = warning;
  warning off
  
  % determine training model
  CurrentSet = find(V ~= i); 
  fKernelPtrTrain = @(X,Y)fKernelPtr(CurrentSet(X),CurrentSet(Y));
  if(~isempty(opt))
        [vModels,vUnique] = core_pvm_mc(fKernelPtrTrain,L(CurrentSet),opt(CurrentSet),C);
  else
        [vModels,vUnique] = core_pvm_mc(fKernelPtrTrain,L(CurrentSet));
  end
  sModel = struct('Models',{vModels},'Labels',{vUnique},'V',{V});  
  warning(s);
  
  % classify test data
  CurrentTestSet = find(V == i); 
  CurrentFullSet = [CurrentSet CurrentTestSet]; % thats only to have order train : test
  fKernelPtrTest = @(X,Y)fKernelPtr(CurrentFullSet(X),CurrentFullSet(Y));        
  DistCenter     = zeros(numel(vModels), length(CurrentTestSet));
  if(~isempty(opt))
      for(k=1:numel(vModels))      
            [~,DistCenter(k,:)] = core_pvm_classify(vModels(k),fKernelPtrTest,[numel(CurrentSet)+1:N],opt(CurrentFullSet));
      end
  else
      for(k=1:numel(vModels))      
            [~,DistCenter(k,:)] = core_pvm_classify(vModels(k),fKernelPtrTest,[numel(CurrentSet)+1:N]);
      end      
  end
  PredictedLabels = argmin(DistCenter); 
  dAcc = sum(vUnique(PredictedLabels)==L(CurrentTestSet))./numel(CurrentTestSet);
  vClassifyResults = [vClassifyResults dAcc];
  vModelResults    = [vModelResults sModel];  
end
